#if defined(ARDUINO_NICLA)

#include "BoschSensortec.h"
#include "BoschParser.h"

BoschSensortec::BoschSensortec() : _acknowledgment(SensorNack),
                                   _debug(NULL)
{
}

BoschSensortec::~BoschSensortec()
{
}

bool BoschSensortec::begin()
{
    auto ret = bhy2_init(BHY2_SPI_INTERFACE, bhy2_spi_read, bhy2_spi_write, bhy2_delay_us, MAX_READ_WRITE_LEN, NULL, &_bhy2);
    if (_debug)
        _debug->println(get_api_error(ret));
    if (ret != BHY2_OK)
        return false;

    bhy2_soft_reset(&_bhy2);

    // Print bhi status
    uint8_t stat = 0;
    //delay(1000);
    ret = bhy2_get_boot_status(&stat, &_bhy2);
    if (_debug)
    {
        _debug->println(get_api_error(ret));
        _debug->print("Boot status: ");
        _debug->println(stat, HEX);
    }

    ret = bhy2_boot_from_flash(&_bhy2);
    if (_debug)
        _debug->println(get_api_error(ret));
    if (ret != BHY2_OK)
        return false;

    ret = bhy2_get_boot_status(&stat, &_bhy2);
    if (_debug)
    {
        _debug->println(get_api_error(ret));
        _debug->print("Boot status: ");
        _debug->println(stat, HEX);
    }

    ret = bhy2_get_host_interrupt_ctrl(&stat, &_bhy2);
    if (_debug)
    {
        _debug->println(get_api_error(ret));
        _debug->print("Interrupt ctrl: ");
        _debug->println(stat, HEX);
    }

    ret = bhy2_get_host_intf_ctrl(&stat, &_bhy2);
    if (_debug)
    {
        _debug->println(get_api_error(ret));
        _debug->print("Interface ctrl: ");
        _debug->println(stat, HEX);
    }

    ret = bhy2_get_and_process_fifo(_workBuffer, WORK_BUFFER_SIZE, &_bhy2);
    if (_debug)
        _debug->println(get_api_error(ret));

    // All sensors' data are handled in the same generic way
    for (uint8_t i = 1; i < BHY2_SENSOR_ID_MAX; i++)
    {
        bhy2_register_fifo_parse_callback(i, BoschParser::parseData, NULL, &_bhy2);
    }

    bhy2_update_virtual_sensor_list(&_bhy2);
    bhy2_get_virt_sensor_list(_sensorsPresent, &_bhy2);

    return true;
}

void BoschSensortec::configureSensor(SensorConfigurationPacket &config)
{
    auto ret = bhy2_set_virt_sensor_cfg(config.sensorId, config.sampleRate, config.latency, &_bhy2);
    if (_debug)
        _debug->println(get_api_error(ret));
    if (ret == BHY2_OK)
    {
        _acknowledgment = SensorAck;
    }
    else
    {
        _acknowledgment = SensorNack;
    }

    if (_config_callback) {
        _config_callback(&config);
    }
}

// acknowledgment flag is reset when read
uint8_t BoschSensortec::acknowledgment()
{
    uint8_t ack = _acknowledgment;
    // Reset acknowledgment
    _acknowledgment = SensorNack;
    return ack;
}

void BoschSensortec::update()
{
    if (get_interrupt_status())
    {
        auto ret = bhy2_get_and_process_fifo(_workBuffer, WORK_BUFFER_SIZE, &_bhy2);
        if (_debug)
            _debug->println(get_api_error(ret));
    }
}

void BoschSensortec::set_config_callback(void (*callback)(SensorConfigurationPacket *)) {
    _config_callback = callback;
}

void BoschSensortec::debug(Stream &stream)
{
    _debug = &stream;
}

#ifdef __cplusplus
extern "C"
{
#endif /*__cplusplus */

#if BHY2_CFG_DELEGATE_FIFO_PARSE_CB_INFO_MGMT
void bhy2_get_fifo_parse_callback_info_delegate(uint8_t sensor_id,
                                                struct bhy2_fifo_parse_callback_table *info,
                                                const struct bhy2_dev *dev)
{
    info->callback_ref = NULL;
    if (sensor_id < BHY2_SENSOR_ID_MAX)
    {
        info->callback = BoschParser::parseData;
    }
    else
    {
        info->callback = NULL;
    }
}
#endif

#ifdef __cplusplus
}
#endif /*__cplusplus */

BoschSensortec sensortec;

#endif